/** ----------------------------------- -o-
 *  StepSwing.java
 *
 *  Sweep back and forth through a range of floating point values,
 *  moving a given size size between min and max.  Direction may change
 *  before and endpoint it reached.  
 *
 *  Occasionaly the delay time transitions from one (1) to double the
 *  given delay time.  Fibonacci sequence measures out the burst and
 *  receed.
 * 
 * INPUT-- 
 *   Atom[] per USAGE
 * RETURN--  
 *   delay time
 * MESSAGES-- 
 *   <constructor> 	defines initial context
 *   algorithm 		one iteration of StepSwing
 *
 * XXX  Use this class as a template for other timer-process algorithms
 *
 * @version 0.1    CREATED: Thu Sep 14 00:00:00 PDT 2009
 * @author  dreeder@danaprajna.com
 */							// -o-
package com.danaprajna;

import java.util.Vector;
import com.cycling74.max.*; 		


public class StepSwing extends MaxObject
{ 							
  //----------------------------------- --o-
  // static variables 			
  //
  private DebugLabel 	z      = new DebugLabel("StepSwing", true);
  private String 	USAGE  = "Usage: StepSwing() "
    + "<parent> <idx> <min> <max> <stepsize> <delay>";

  private ToolBox   t 	     = new ToolBox();
  private Automata  parent   = null;  	// output channel to patch environment
  private int 	    threadId = -1;



  //----------------------------------- --o-
  // constructors (and argument parse) 
  //
  public StepSwing(Automata p, int idx, Atom[] a)  
  {
    z.st(idx).arglist("StepSwing", a).ms(); 	//.b(a);

    if (null==p)      { z.merr("parent is undefined"); 		     return; }
    if (0>idx)        { z.merr("<idx> is less than zero"); 	     return; }
    if (4!=a.length)
      		      { z.merr("<args> is of the wrong length");     return; }
    if ( !(a[0].isFloat() && a[1].isFloat() 
	      && a[2].isFloat() && a[3].isInt()) )
      		      { z.merr("some <args> are of the wrong type"); return; }

    parent    = p;
    threadId  = idx;  

    min 	= a[0].getFloat();
    max 	= a[1].getFloat();
    stepsize 	= a[2].getFloat();
    delayRef 	= a[3].getInt();


    if (stepsize >= (max-min)) { 
        z.merr("<stepsize> must be < (max-min)  (" + (max-min) + ")"); 
								     return; }
    if (0 >= delayRef) 	
      { z.merr("<delay> must be greater than zero"); 		     return; }


    // initialize
    //
    step = (max-min) / 2.;
    interruptTrigger = t.getRand(interruptTriggerField);

    // z.e();
  }  // public StepSwing(Atom[])


  protected void notifyDeleted()  { z.sm("notifyDeleted").d().ms(); }
  
  public void start()  { z.sm("start").d().ms(); return; }
  public void stop()   { z.sm("stop").d().ms(); return; }



  //----------------------------------- --o-
  // built-in messages 					
  //
  public void usage() { usage(true); }
  public void usage(boolean do_longversion) {
    String space = "\t\t\t\t\t";
    z.pc(USAGE); if (false == do_longversion) { return; } 

    z.pc(space + "min \t\t\t lowest value of sweep");
    z.pc(space + "max \t\t\t highest value of sweep");
    z.pc(space + "stepsize \t\t\t granularity of increment");
    z.pc(space + "delay \t\t\t delay reference value");
  }



  //----------------------------------- --o-
  // algorithm 						
  //

  private double min 	  = 0.; 	// low end of swing range
  private double max 	  = 0.; 	// high end of swing range
  private double stepsize = 0.; 	// step size of change within range

  private int delayRef   = 0; 		// reference delay from input parameters
  private int delayReal  = 0; 		// actual current delay value

  private int     direction  = 1; 	// count up vs. down
  private double  step       = 0.; 	// current position between min and max
    

  private boolean interrupt 		= false; 
      // true: use fib sequence for delay time

  private int     interruptTriggerField = 17; 		
      // size of a field of random numbers generated by Urn() 

  private int     interruptTrigger 	= -1; 
      // chosen from a field, size N, of random numbers.
      // signals interrupt at that point while traversing the field.

  private Urn     u 			= new Urn(interruptTriggerField);
  private int 	  urn_value 		= u.next();
  private int     fibIndex 		= 1;

      

  public double algorithm() { 
    //z.b("algorithm");

    // computer current step in range [min, max]
    //
    if (1==t.getRand(100))  { direction *= -1; }
    step += stepsize * direction; 		// %1 chance direction changes

    if      (step<min)  { step = min; direction = 1; } 	
    else if (step>max)  { step = max; direction = -1; }


    // begin interrupt sequence
    //
    if (interrupt) { 				
      if ( delayReal < (2*delayRef) ) { 	
        delayReal = t.fib(fibIndex++); 		// delay <- fibonnaci

	  // XXX  what else might be used here?
	  // XXX  how else to manage shift around delayRef?

      } else { 					// reset delay time
        delayReal = delayRef;
	fibIndex  = 1;
	interrupt = false;
      }

    } else {
      urn_value = u.next(); 		// next step in random field

      if (u.cycleRestarted)  		// reset trigger at field boundaries
        { interruptTrigger = t.getRand(interruptTriggerField); }

      if (interruptTrigger==urn_value)  { interrupt = true; }
      					// test for next interrupt
    }


    parent.out(threadId, step);

    //z.e((double) delayReal);
    return((double) delayReal);
  }  // algorithm()

  

  //----------------------------------- --o-
  // something risque 			
  // 
  public void experiment(Atom[] args) { 
    z.arglist("experiment", args);  //z.b("experiment");
    /*EMPTY*/
    //z.e();
  }  // experiment()

  public void experiment()  { Atom[] a = null; experiment(a); }

}  // public class StepSwing extends MaxObject

